#include <IRremote.h>
//#include <avr/sleep.h>
#include <avr/wdt.h>

//IR LED on pin D3
#define LED 13
#define B0  2  //power
#define B1  4  //up
#define B2  5  //left
#define B3  6  //center
#define B4  7  //right
#define B5  8  //down
#define B6  9  //ru
#define B7  10 //rd

//#define USE_OPEN_DRAIN_OUTPUT_FOR_SEND_PIN
IRsend irsend;

//https://tasmota.github.io/docs/Codes-for-IR-Remotes/
unsigned long tmp = 0;
const unsigned int bits = 12;
unsigned long power_cmd = 0xA90;
unsigned long play_cmd  = 0b010011010001;
unsigned long pause_cmd = 0b100111010001;
unsigned long input_cmd = 0xA50;
unsigned long chup_cmd  = 0x090;
unsigned long chdn_cmd  = 0x890;
unsigned long volup_cmd = 0x490;
unsigned long voldn_cmd = 0xC90;

ISR(PCINT0_vect){
   PCIFR&=0b11111110;
}

ISR(PCINT1_vect){
   PCIFR&=0b11111101;
}

ISR(PCINT2_vect){
   PCIFR&=0b11111011;
}

void config_slp(void){
  wdt_disable();  //disable watchdog
  ADCSRA &= ~(1<<7);   //ADEN=0, turn off adc
  PCMSK0=0b00010111;
  PCMSK1=0b00111111; //turn on individual pin change interrupts for all button inputs
  PCMSK2=0b11110100;
  PCIFR=0x00; //clear all interrupt flags
  PCICR=0b00000111; //turn on interrupts for pin changes on three banks of inputs
  SMCR=0b00000101;  //enable sleeping, power down mode
  PRR=0b10000111; //power down individual peripherals to lower current consumption, check if shutting off timers will break irsend!!!!!
}

void setup() {
  pinMode(B0,INPUT_PULLUP);
  pinMode(B1,INPUT_PULLUP);
  pinMode(B2,INPUT_PULLUP);
  pinMode(B3,INPUT_PULLUP);
  pinMode(B4,INPUT_PULLUP);
  pinMode(B5,INPUT_PULLUP);
  pinMode(B6,INPUT_PULLUP);
  pinMode(B7,INPUT_PULLUP);
  pinMode(LED,OUTPUT);

  pinMode(A0,INPUT);  //IR recv data
  pinMode(A1,INPUT);  //IR recv Vcc

  config_slp();
}

void loop() {
  if(digitalRead(B0)==0){tmp=power_cmd;
  }else if(digitalRead(B1)==0){tmp=volup_cmd;
  }else if(digitalRead(B2)==0){tmp=chdn_cmd;
  }else if(digitalRead(B3)==0){tmp=play_cmd;
  }else if(digitalRead(B4)==0){tmp=chup_cmd;
  }else if(digitalRead(B5)==0){tmp=voldn_cmd;
  }else if(digitalRead(B6)==0){tmp=input_cmd;
  }else if(digitalRead(B7)==0){tmp=pause_cmd;
  }else{tmp=0;}

  if(tmp!=0){
    for (int i = 0; i < 3; i++) {
      digitalWrite(LED,HIGH);
      irsend.sendSony(tmp, bits); //send ir command/address
      //IrSender.enableIROut(38);
      //IrSender.sendNEC(0x4 & 0x1F, tmp & 0x7F, 0); //send ir command/address
      //IrSender.end();
      //IrSender.IRLedOff();
      //pinMode(3,INPUT);
      delay(40);
    }
    digitalWrite(LED,LOW);
    delay(250); //delay between each signal burst
    tmp=0;
  }

  asm("sei"); //enable global interrupts
  //sleep_bod_disable();
  MCUCR |= (3<<5); //set both BODS and BODSE at the same time
  MCUCR = (MCUCR & ~(1<<5))|(1<<6); //then set BODS bit and clear BODSE bit at the same time
  asm("sleep"); //go to sleep, check power consumption in sleep!!!!
}

/*void loop() {
      irsend.sendSony(0xA90,12); // Sony TV power code
      irsend.sendSony(0x490,12); // Sony TV vol +
      irsend.sendSony(0x890,12); // Sony TV channel -
      irsend.sendSony(0x37EE,15); // Sony TV favorite
      irsend.sendSony(0x090,12); // Sony TV channel +
      irsend.sendSony(0xC90,12); // Sony TV vol -
      irsend.sendSony(0xA50,12); // Sony TV input
      irsend.sendSony(0xF50,12); // Sony TV power off
}*/
